package benefactoraddressvalidator

import (
	"errors"

	param "git.gocasts.ir/ebhomengo/niki/param/benefactor/address"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
	validation "github.com/go-ozzo/ozzo-validation/v4"
)

func (v Validator) ValidateUpdateAddress(req param.UpdateAddressRequest) (map[string]string, error) {
	const op = "benefactoraddressvalidator.ValidateUpdateAddress"

	if req.PostalCode == "" && req.Address == "" && req.Name == "" &&
		req.Lat == 0 && req.Lon == 0 && req.CityID == 0 {

		return map[string]string{"update": "at least one field should be provided"}, richerror.New(op).
			WithMessage(errmsg.ErrorMsgInvalidInput).
			WithKind(richerror.KindInvalid).
			WithMeta(map[string]interface{}{"req": req}).
			WithErr(errors.New("at least one field should be provided"))
	}

	if err := validation.ValidateStruct(&req,
		validation.Field(&req.BenefactorID, validation.Required,
			validation.By(v.doesBenefactorExist)),

		validation.Field(&req.ID, validation.Required,
			validation.By(v.doesAddressExist(req.BenefactorID))),

		validation.Field(&req.CityID,
			validation.When(req.CityID != 0, validation.By(v.doesCityExist))),
	); err != nil {
		fieldErrors := make(map[string]string)

		var errV validation.Errors
		if errors.As(err, &errV) {
			for key, value := range errV {
				if value != nil {
					fieldErrors[key] = value.Error()
				}
			}
		}

		return fieldErrors, richerror.New(op).
			WithMessage(errmsg.ErrorMsgInvalidInput).
			WithKind(richerror.KindInvalid).
			WithMeta(map[string]interface{}{"req": req}).
			WithErr(err)
	}

	return map[string]string{}, nil
}
